﻿//////////////////////////////////////////////
// CompositorNode.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Compositor ;
	class TargetOperations ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT CompositorNode : public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Les opérations à rendre sur les targets
			std::vector<TargetOperations*> _operationsList ;

			// Infos optionnelles de debug
			std::string _name ;
			Compositor* _parentCompositor ;

			// Actif ou pas
			bool _active ;
		
		public :
		
			// Constructeur, destructeur
			CompositorNode () ;
			virtual ~CompositorNode () ;

			// Concernant les TargetOperations
			virtual TargetOperations* addOperations () = 0 ;
			TargetOperations* getOperations (unsigned int index) const ;
			unsigned int getOperationsCount () const ;
			virtual void deleteOperations (unsigned int index) ;
			virtual void changeOperationsOrder (unsigned int fromIndex, unsigned int toIndex) ;

			// Getters
			nkMemory::StringView getName () const ;
			Compositor* getParentCompositor() const ;
			bool getActive () const ;
				
			// Setters
			void setName (const nkMemory::StringView& name) ;
			void setParentCompositor (Compositor* value) ;
			virtual void setActive (bool value) ;

			// Rendu
			virtual void renderOperations () ;

			// L'export et import
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}