﻿//////////////////////////////////////////////
// InputSystem.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	struct Point ;
}

/// Includes ---------------------------------

// nkWinUi
#include "CallbackRepository.h"
#include "EventTypes.h"

#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <map>
#include <vector>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT InputSystem final : public nkCommon::SingletonClass<InputSystem>
	{
		friend class nkCommon::SingletonClass<InputSystem> ;

		private :
		
			// Constructeur, destructeur
			InputSystem () ;
			~InputSystem () ;

			// Les callbacks
			mouseClickCallback _mouseClickCallback ;
			mouseLeaveCallback _mouseLeaveCallback ;
			keyDownCallback _keyDownCallback ;
			keyUpCallback _keyUpCallback ;

			menuBarCallback _menuBarCallback ;

			checkBoxCallback _checkBoxCallback ;
			treeSelectionCallback _treeViewCallback ;
			listBoxSelectionCallback _listBoxCallback ;
			comboSelectionCallback _comboCallback ;
			editBoxKeyCallback _editBoxCallback ;

			closeCallback _closeCallback ;

			contextMenuCreationCallback _contextMenuCreationCallback ;

			// Mémoire pour les mouvements relatifs souris
			int _lastMouseX ;
			int _lastMouseY ;

			// Les composants ayant demandé de recevoir des évènement sans être focus
			std::map<EVENT_TYPE, std::vector<Component*>> _trackedComponents ;

			// Mémoire des fenêtres suivies pour les sorties souris
			std::map<Component*, std::pair<bool, bool>> _followedWindows ;
		
		public :			

			// Demande de tracking d'events général
			void registerToTrack (EVENT_TYPE eventType, Component* component) ;
			void unregisterTracking (EVENT_TYPE eventType, Component* component) ;
			// Demande de tracking de fenêtre (souris qui sort)
			void requestWindowFollow (Component* caller, bool value) ;

			// Getters
			mouseClickCallback getMouseClickCallback () const ;
			keyDownCallback getKeyDownCallback () const ;
			keyUpCallback getKeyUpCallback () const ;

			menuBarCallback getMenuBarCallback () const ;
			closeCallback getCloseCallback () const ;
			contextMenuCreationCallback getContextMenuCreationCallback () const ;
			const std::map<Component*, std::pair<bool, bool>>& getFollowedWindows () const ;

			// Setters des fonctions de callback
			void setMouseClickCallback (mouseClickCallback callback) ;
			void setMouseLeaveCallback (mouseLeaveCallback callback) ;
			void setKeyDownCallback (keyDownCallback callback) ;
			void setKeyUpCallback (keyUpCallback callback) ;

			void setMenuCallback (menuBarCallback callback) ;
			void setCheckBoxCallback (checkBoxCallback callback) ;
			void setTreeSelectionCallback (treeSelectionCallback callback) ;
			void setListBoxSelectionCallback (listBoxSelectionCallback callback) ;
			void setComboSelectionCallback (comboSelectionCallback callback) ;
			void setEditBoxKeyCallback (editBoxKeyCallback callback) ;

			void setCloseCallback (closeCallback callback) ;

			void setContextMenuCreationCallback (contextMenuCreationCallback callback) ;

			// Changement position curseur
			void setCursorPositionScreen (const nkMaths::Point& absPos) ;
	} ;
}